
function [x2,x3,y,y2,sol,sol1]=PDE_ellittiche_condotto(x0,xN,h,f,g0,g1,g2);
%_____________________________________________________________________________
%
% Funzione PDE_ellittiche_condotto
% ____________________________________
%
% Chiamata:
% [x2,x3,y,y2,sol,sol1]=PDE_ellittiche_condotto(x0,xN,h,f,g0,g1,g2);
%
% Commento:
%
% La funzione 'PDE_ellittiche_condotto' utilizza le formule alle differenze finite 
% per approssimare Uxx e Uyy, e calcolare la soluzione del seguente problema:
%
% - Uyy - Uxx = F(x,y)    x0<x<xN, y0<y<yM 
%
% che rappresenta il flusso di un fluido incomprimibile in un canale con
% dominio a L dove le condizioni sono assegnate su:            
%
%          /    g0(y)        y0<=y<=yM, x=x0     
% U(x,y)= |     g1(x)        x0<=x<=xN, y=y0    
%          \    g2(x)        x0<=x<=2*xN, y=yM    
% 
% Data la simmetria del dominio  possibile risolvere il problema sulla prima 
% met rispetto all'asse di simmetria.  
% Si ricorda che x0=y0 e yM=xN.
% dove: 
% g0(x), g1(y), g2(x) sono le condizioni assegnate sul bordo.
%                                                                    
% NB: nel programma k  posto uguale a h che va preso della larghezza del condotto
% per non perdere la simmetria. 
%_____________________________________________________________________________
% 
%
% Parametri di input della function:
%
% x0,xN  estremi dell'intervallo [x0,xN] di integrazione relativo all'asse x;         
% h      passo di discretizzazione sull'asse x, ossia ampiezza dei sottointervalli
%        in cui e' stato suddiviso [x0,xN]; esso coincide con la suddivisione
%        di [y0,yM];
% f      stringa contenente l'espressione della funzione f(x,y);
% g0     stringa contenente l'espressione della funzione g0(y), che rappresenta 
%        le condizioni assegnate al bordo sinistro del dominio;
% g1,g2  stringhe contenenti l'espressione delle funzioni g1(x) e g2(x), che  
%        rappresentano le condizioni rispettivamente ai bordi inferiore e  
%        superiore del dominio;
%
% Parametri di output della function:
%
% x2   vettore colonna di dimensione 2(M+2)-1 contenente i nodi x0,x1,...,2*xN;
% x3   vettore colonna di dimensione M+2 contenente i nodi xN,xN+1,...,2*xN;
% y    vettore colonna di dimensione M+2 contenente i nodi y0,y1,...,yM;
% y2   vettore colonna di dimensione M+2 contenente i nodi -yM,...,y0;
% sol1 contiene la soluzione in [1,2]x[-1,0] (x3 x y2)
% sol  contiene la soluzione in [0,2]x[0 ,1] (x2 x y)
% La soluzione  quindi contenuta in due matrici.
%__________________________________________________________________________
y0=x0;
yM=xN;
y =(y0:h:yM)';    y(end)=yM;
x1=(x0:h:xN)';   x1(end)=xN;
x2=(x0:h:2*xN)'; x2(end)=2*xN;
x3=(xN:h:2*xN)'; x3(end)=2*xN;
y2=-xN:h:y0;   y2(end)=y0;
M=length(y)-2;        % Numero di punti in cui  stata partizionata la sezione del condotto
s=M*(M+1)+sum([1:M]); % Dimensione della matrice dei coefficienti del sistema lineare
%______________________________________________
%
% costruzione del vettore termini noti
%______________________________________________
% 
f=[f,'+0*x+0*y']; f=inline(f);
[X,Y]=meshgrid(x2(2:end-1),y(2:end-1));
Z1=feval(f,X,Y);
Z=Z1(:);
ind=(M+2)*M;  
F=Z(1:ind);
for j=1:M-1
    ind1=(M+1)*(M+j);   
    F=[F;  Z(ind1+1:ind1+M-j)];
end
%    
% il vettore G tiene conto delle condizioni al contorno
%
g0=[g0,'+0*y']; g0=inline(g0);
g1=[g1,'+0*x1']; g1=inline(g1);
g2=[g2,'+0*x2']; g2=inline(g2);
g0=feval(g0,y);
g1=feval(g1,x1);
g2=feval(g2,x2);
% condizioni al bordo sinistro g0
G=zeros(1,s);
G(1:M)=g0(2:M+1);
G(1)=G(1)+g1(2);
G(M)=G(M)+g2(2);
% condizioni ai bordi superiore ed inferiore
for j=2:M+1
     G((j-1)*M+1)=g1(j); %sotto g1
     G(j*M)=g2(j);  %sopra g2
end
ind=(M+1)*M;
% condizioni al bordo destro g2
for j=M:-1:1
    ind=ind+j;
    G(ind)=g2(2*M+3-j);
end
G(end)=G(end)+g2(end);

b=h^2*F+G';
%______________________________________________
%
% Costruzione della matrice dei coefficienti A
%______________________________________________
% costruzione M-esima codiagonale inferiore in "I_M"
e1=-ones(M,1);
I_M=[]; % zeros(M,1);
for i=1:M
    I_M=[I_M; e1];
end
L=sum(1:M-1);
I_M=[I_M; [-2; -ones(M-1,1)]; zeros(L,1); zeros(M,1)];
% costruzione delle (M-1)-esima, (M-2)-esima ... 2-esima codiagonali
% inferiori in "mat"
mat=[];
ind=M*(M+1)+1;
for i=M-1:-1:2
    e_i=[0; -2; -ones(i-1,1)];
    L=length(e_i);
    I=zeros(s,1);
    I(ind:ind+L-1)=e_i;
    ind=ind+i+1;
    mat=[mat I];
end
% costruzione codiagonale inferiore in "I_1"
e1=[-ones(M-1,1); 0];
I_1=[];
for i=1:M+1
    I_1=[I_1; e1];
end
 for i=M-1:-1:2
     ei=[-ones(i,1) ; 0 ];
     I_1=[I_1; ei];
 end
 I_1=[I_1; -1 ; -2;0];

% costruzione diagonale principale della matrice A in "I0"
I0=4*ones(s,1);  
% costruzione codiagonale superiore in "I1"
e1=[-ones(M-1,1); 0];
I1=[0];
for i=1:M+1
    I1=[I1; e1];
end
 for i=M-1:-1:2
     ei=[-2; -ones(i-1,1) ; 0 ];
     I1=[I1; ei];
 end
 I1=[I1; [-2; -1]];

% costruzione delle (M-1)-esima, (M-2)-esima ... 2-esima codiagonali superiori in  "mat1"
mat1=[];
ind=M*(M+2);
for i=M-2:-1:1
    ei=[0 ; -ones(i+1,1)];
    L=length(ei);
    I=zeros(s,1);
    I(ind:ind+L-1)=ei;
    ind=ind+i+1;
    mat1=[I mat1 ];
end
% costruzione M-esima codiagonale superiore in "IM"
e1=-ones(M,1);
IM=[];
for i=1:M+1
    IM=[IM; e1];
end
L=sum(1:M-1);
IM=[zeros(M,1); IM; zeros(L,1)];
diag_A=[I_M mat I_1 I0 I1 mat1 IM]
A = spdiags(diag_A, -M:M , s, s);

%__________________________
%
% calcolo della soluzione
%__________________________
U=A\b; %sceglie il mtodo pi adatto di soluzione
%
%la soluzione trovata viene messa in due matrici  sol1 e sol
%
for j=1:M+1
    sol(:,j)=[g1(j+1) U((j-1)*M+1:j*M)' g2(j+1)]';
end
sol=[g0 sol];  % soluzione in [0,1]x[0,1] (x1 x y)
%---------
mat3=[];
ind=(M+1)*M;
for j=M:-1:1
    ei=zeros(M-j,1);
    mat3=[mat3 [ ei; U(ind+1:ind+j)]];
    ind=ind+j;
end
mat4=mat3';
mat4=mat3+triu(mat4,1);
mat4=[mat4; g2(M+3:2*M+2)' ];  % condizione bordo superiore
mat4=[mat4 g2(2*M+3:-1:M+3)] ; % condizione bordo destro
mat4=[sol(2:end,end)';mat4];   % soluzione in [1,2]x[0,1]: x3 x y
%---------
sol1=sol';                     % soluzione in [1,2]x[-1,0]: x3 x y2
sol=[sol mat4];                 % soluzione in [0,2]x[0 ,1]: x2 x y   
